<?php
require("HttpClient.class.php");

/**
 * 微信公共平台的私有接口
 * 思路: 模拟登录, 再去调用私有web api
 *
 * 功能: 发送信息, 批量发送, 获取用户信息, 获取最近信息
 *
 * @author zhaoxianlie
 *
 * 有问题，可以到 http://www.baidufe.com 进行反馈
 *
 * 参考了gitHub微信的api: https://github.com/lealife/WeiXin-Private-API, 在此基础上作了修改和完善
 */
class WeiXin {

    // 公众平台账号
	private $account;
    // 公众平台密码
	private $password;

    // 保存cookie的文件路径
	private $cookiePath;
    // 保存webToken的文路径，注：webToken是每次请求中携带的动态token
	private $webTokenPath;

	private $webToken;
	private $cookie;

    // HttpClient对象，用来发送Get或Post请求
	private $httpClient;

    /**
     * 在构造函数中完成一系列初始化
     * @param $config
     */
    public function __construct($config) {
		if(!$config || !is_array($config)) {
			exit("weixin arguments invalid");
		}

		// 配置初始化
		$this->account = $config['account'];
		$this->password = $config['password'];
		$this->cookiePath = $config['cookiePath'];
		$this->webTokenPath = $config['webTokenPath'];

		$this->httpClient = new HttpClient();

		// 读取cookie, webToken
		$this->getCookieAndWebToken();
	}

    /**
     * 获取mp.weixin.qq.com的referer
     * @param array $data
     * @return string
     */
    private function getMpWxReferer($data = array()){
        return "https://mp.weixin.qq.com/cgi-bin/singlesendpage"
            . "?t=message/send"
            . "&action=index"
            . "&tofakeid={$data['tofakeid']}"
            . "&token={$data['token']}"
            . "&lang=zh_CN";
    }

    /**
     * 获取web微信的referer
     * @return string
     */
    private function getWebWxReferer(){
        return "https://wx.qq.com/";
	}


	/**
	 * 模拟登录获取cookie和webToken
	 */
	public function login() {
		$url = "https://mp.weixin.qq.com/cgi-bin/login?lang=zh_CN";
		$post["username"] = $this->account;
		$post["pwd"] = md5($this->password);
        $post["imgcode"] = "";
		$post["f"] = "json";
		$re = $this->httpClient->post($url, $post);

		// 得到cookie
		$this->getCookie($re['cookie']);

		// 得到token
		$this->getWebToken($re['body']);

		return true;
	}

    /**
     * 登录后从结果中得到cookie
     * @param $loginRet
     */
    private function getCookie($cookie = "") {
        if(!empty($cookie)) {
            $this->cookie = $cookie;
            file_put_contents($this->cookiePath, $cookie);
            return true;
        }else{
            return false;
        }
    }

    /**
     * 登录后从结果中解析出webToken
     * @param  [String] $loginRet
     * @return [Boolen]
     */
    private function getWebToken($loginRet) {
        $loginRet = json_decode($loginRet, true);
        $msg = $loginRet["redirect_url"];
        $msgArr = explode("&token=", $msg);
        if(count($msgArr) != 2) {
            return false;
        } else {
            $this->webToken = $msgArr[1];
            file_put_contents($this->webTokenPath, $this->webToken);
            return true;
        }
    }

	/**
	 * 从缓存中得到cookie和webToken
	 * @return [type]
	 */
	public function getCookieAndWebToken() {
        // 如果并发度不高，所以每次都默认强制login一次，要不然cookie和webtoken可能会失效
        return $this->login();
	}

	/**
	 * 主动发消息
	 * @param  string $id      用户的fakeid
	 * @param  string $content 发送的内容
	 * @return [type]          [description]
	 */
	public function send($id, $content) {
		$post = array();
		$post['tofakeid'] = $id;
		$post['type'] = 1;
		$post['content'] = $content;
		$post['ajax'] = 1;
        $post['token'] = $this->webToken;
        $post['t'] = "ajax-response";
        $post['f'] = "json";
        $post['random'] = Math.rand(0,1);
        $post['lang'] = 'zh_CN';
        $post['imgcode'] = '';
		$url = "https://mp.weixin.qq.com/cgi-bin/singlesend";
		$re = $this->httpClient->post($url, $post, $this->cookie);
		return json_decode($re['body']);
	}

	/**
	 * 批量发送
	 * @param  [array] $ids     用户的fakeid集合
	 * @param  [type] $content [description]
	 * @return [type]          [description]
	 */
	public function batSend($ids, $content) {
		$result = array();
		foreach($ids as $id) {
			$result[$id] = $this->send($id, $content);
		}
		return $result;
	}	

	/**
	 * 发送图片
	 * @param  int $fakeId [description]
	 * @param  int $fileId 图片ID
	 * @return [type]         [description]
	 */
	public function sendImage($fakeId, $fileId) {
		$post = array();
		$post['tofakeid'] = $fakeId;
		$post['type'] = 2;
		$post['fid'] = $post['fileId'] = $fileId; // 图片ID
		$post['error'] = false;
		$post['ajax'] = 1;
		$post['token'] = $this->webToken;
        $post['lang'] = 'zh_CN';
        $post['t'] = "ajax-response";

		$url = "https://mp.weixin.qq.com/cgi-bin/singlesend";
		$re = $this->httpClient->post($url, $post, $this->cookie);

		return json_decode($re['body']);
	}

	/**
	 * 获取用户的信息
	 * @param  string $fakeId 用户的fakeId
	 * @return [type]     [description]
	 */
	public function getUserInfo($fakeId) {
		$url = "https://mp.weixin.qq.com/cgi-bin/getcontactinfo"
            . "?t=ajax-getcontactinfo"
            . "&lang=zh_CN"
            . "&token={$this->webToken}"
            . "&fakeid=$fakeId";
		$re = $this->httpClient->get($url, $this->cookie);
		$result = json_decode($re['body'], 1);
		if(!$result) {
			$result = '{"errmsg":"获取失败，可能是未登录"}';
		}
		return $result;
	}

    /**
     * 得到最近发来的信息
     * @param int $page
     * @return mixed|string
     */
    public function getLatestMsgs($page = 0) {
		$offset = 50 * $page;
		$url = "https://mp.weixin.qq.com/cgi-bin/message"
            . "?t=message/list"
            . "&count=999999"
            . "&day=7"
            . "&offset={$offset}"
            . "&token={$this->webToken}"
            . "&lang=zh_CN";
		$re = $this->httpClient->get($url, $this->cookie);

		// 解析得到数据
		$match = array();
		preg_match('/["\' ]msg_item["\' ]:\[{(.+?)}\]/', $re['body'], $match);
		if(count($match) != 2) {
			return "";
		}

		$match[1] = "[{". $match[1]. "}]";
		return json_decode($match[1], 1);
	}
}
